#!/usr/bin/python
from __future__ import absolute_import, division, print_function
# Copyright 2019-2020 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fmgr_system_saml
short_description: Global settings for SAML authentication.
description:
    - This module is able to configure a FortiManager device.
    - Examples include all parameters and values which need to be adjusted to data sources before usage.

version_added: "2.10"
author:
    - Link Zheng (@chillancezen)
    - Jie Xue (@JieX19)
    - Frank Shen (@fshen01)
    - Hongbin Lu (@fgtdev-hblu)
notes:
    - Running in workspace locking mode is supported in this FortiManager module, the top
      level parameters workspace_locking_adom and workspace_locking_timeout help do the work.
    - To create or update an object, use state present directive.
    - To delete an object, use state absent directive.
    - Normally, running one module can fail when a non-zero rc is returned. you can also override
      the conditions to fail or succeed with parameters rc_failed and rc_succeeded

options:
    bypass_validation:
        description: only set to True when module schema diffs with FortiManager API structure, module continues to execute without validating parameters
        required: false
        type: bool
        default: false
    workspace_locking_adom:
        description: the adom to lock for FortiManager running in workspace mode, the value can be global and others including root
        required: false
        type: str
    workspace_locking_timeout:
        description: the maximum time in seconds to wait for other user to release the workspace lock
        required: false
        type: int
        default: 300
    state:
        description: the directive to create, update or delete an object
        type: str
        required: true
        choices:
          - present
          - absent
    rc_succeeded:
        description: the rc codes list with which the conditions to succeed will be overriden
        type: list
        required: false
    rc_failed:
        description: the rc codes list with which the conditions to fail will be overriden
        type: list
        required: false
    system_saml:
        description: the top level parameters set
        required: false
        type: dict
        suboptions:
            acs-url:
                type: str
                description: 'SP ACS(login) URL.'
            cert:
                type: str
                description: 'Certificate name.'
            entity-id:
                type: str
                description: 'SP entity ID.'
            idp-cert:
                type: str
                description: 'IDP Certificate name.'
            idp-entity-id:
                type: str
                description: 'IDP entity ID.'
            idp-single-logout-url:
                type: str
                description: 'IDP single logout url.'
            idp-single-sign-on-url:
                type: str
                description: 'IDP single sign-on URL.'
            login-auto-redirect:
                type: str
                default: 'disable'
                description:
                 - 'Enable/Disable auto redirect to IDP login page.'
                 - 'disable - Disable auto redirect to IDP Login Page.'
                 - 'enable - Enable auto redirect to IDP Login Page.'
                choices:
                    - 'disable'
                    - 'enable'
            role:
                type: str
                default: 'SP'
                description:
                 - 'SAML role.'
                 - 'IDP - IDentiy Provider.'
                 - 'SP - Service Provider.'
                choices:
                    - 'IDP'
                    - 'SP'
            server-address:
                type: str
                description: 'server address.'
            service-providers:
                description: no description
                type: list
                suboptions:
                    idp-entity-id:
                        type: str
                        description: 'IDP Entity ID.'
                    idp-single-logout-url:
                        type: str
                        description: 'IDP single logout url.'
                    idp-single-sign-on-url:
                        type: str
                        description: 'IDP single sign-on URL.'
                    name:
                        type: str
                        description: 'Name.'
                    prefix:
                        type: str
                        description: 'Prefix.'
                    sp-cert:
                        type: str
                        description: 'SP certificate name.'
                    sp-entity-id:
                        type: str
                        description: 'SP Entity ID.'
                    sp-single-logout-url:
                        type: str
                        description: 'SP single logout URL.'
                    sp-single-sign-on-url:
                        type: str
                        description: 'SP single sign-on URL.'
            sls-url:
                type: str
                description: 'SP SLS(logout) URL.'
            status:
                type: str
                default: 'disable'
                description:
                 - 'Enable/disable SAML authentication (default = disable).'
                 - 'disable - Disable SAML authentication.'
                 - 'enable - Enabld SAML authentication.'
                choices:
                    - 'disable'
                    - 'enable'

'''

EXAMPLES = '''
 - hosts: fortimanager-inventory
   collections:
     - fortinet.fortimanager
   connection: httpapi
   vars:
      ansible_httpapi_use_ssl: True
      ansible_httpapi_validate_certs: False
      ansible_httpapi_port: 443
   tasks:
    - name: Global settings for SAML authentication.
      fmgr_system_saml:
         bypass_validation: False
         workspace_locking_adom: <value in [global, custom adom including root]>
         workspace_locking_timeout: 300
         rc_succeeded: [0, -2, -3, ...]
         rc_failed: [-2, -3, ...]
         system_saml:
            acs-url: <value of string>
            cert: <value of string>
            entity-id: <value of string>
            idp-cert: <value of string>
            idp-entity-id: <value of string>
            idp-single-logout-url: <value of string>
            idp-single-sign-on-url: <value of string>
            login-auto-redirect: <value in [disable, enable]>
            role: <value in [IDP, SP]>
            server-address: <value of string>
            service-providers:
              -
                  idp-entity-id: <value of string>
                  idp-single-logout-url: <value of string>
                  idp-single-sign-on-url: <value of string>
                  name: <value of string>
                  prefix: <value of string>
                  sp-cert: <value of string>
                  sp-entity-id: <value of string>
                  sp-single-logout-url: <value of string>
                  sp-single-sign-on-url: <value of string>
            sls-url: <value of string>
            status: <value in [disable, enable]>

'''

RETURN = '''
request_url:
    description: The full url requested
    returned: always
    type: str
    sample: /sys/login/user
response_code:
    description: The status of api request
    returned: always
    type: int
    sample: 0
response_message:
    description: The descriptive message of the api response
    type: str
    returned: always
    sample: OK.

'''
from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.connection import Connection
from ansible_collections.fortinet.fortimanager.plugins.module_utils.napi import NAPIManager
from ansible_collections.fortinet.fortimanager.plugins.module_utils.napi import check_galaxy_version
from ansible_collections.fortinet.fortimanager.plugins.module_utils.napi import check_parameter_bypass


def main():
    jrpc_urls = [
        '/cli/global/system/saml'
    ]

    perobject_jrpc_urls = [
        '/cli/global/system/saml/{saml}'
    ]

    url_params = []
    module_primary_key = None
    module_arg_spec = {
        'bypass_validation': {
            'type': 'bool',
            'required': False,
            'default': False
        },
        'workspace_locking_adom': {
            'type': 'str',
            'required': False
        },
        'workspace_locking_timeout': {
            'type': 'int',
            'required': False,
            'default': 300
        },
        'rc_succeeded': {
            'required': False,
            'type': 'list'
        },
        'rc_failed': {
            'required': False,
            'type': 'list'
        },
        'system_saml': {
            'required': False,
            'type': 'dict',
            'options': {
                'acs-url': {
                    'required': False,
                    'type': 'str'
                },
                'cert': {
                    'required': False,
                    'type': 'str'
                },
                'entity-id': {
                    'required': False,
                    'type': 'str'
                },
                'idp-cert': {
                    'required': False,
                    'type': 'str'
                },
                'idp-entity-id': {
                    'required': False,
                    'type': 'str'
                },
                'idp-single-logout-url': {
                    'required': False,
                    'type': 'str'
                },
                'idp-single-sign-on-url': {
                    'required': False,
                    'type': 'str'
                },
                'login-auto-redirect': {
                    'required': False,
                    'choices': [
                        'disable',
                        'enable'
                    ],
                    'type': 'str'
                },
                'role': {
                    'required': False,
                    'choices': [
                        'IDP',
                        'SP'
                    ],
                    'type': 'str'
                },
                'server-address': {
                    'required': False,
                    'type': 'str'
                },
                'service-providers': {
                    'required': False,
                    'type': 'list',
                    'options': {
                        'idp-entity-id': {
                            'required': False,
                            'type': 'str'
                        },
                        'idp-single-logout-url': {
                            'required': False,
                            'type': 'str'
                        },
                        'idp-single-sign-on-url': {
                            'required': False,
                            'type': 'str'
                        },
                        'name': {
                            'required': False,
                            'type': 'str'
                        },
                        'prefix': {
                            'required': False,
                            'type': 'str'
                        },
                        'sp-cert': {
                            'required': False,
                            'type': 'str'
                        },
                        'sp-entity-id': {
                            'required': False,
                            'type': 'str'
                        },
                        'sp-single-logout-url': {
                            'required': False,
                            'type': 'str'
                        },
                        'sp-single-sign-on-url': {
                            'required': False,
                            'type': 'str'
                        }
                    }
                },
                'sls-url': {
                    'required': False,
                    'type': 'str'
                },
                'status': {
                    'required': False,
                    'choices': [
                        'disable',
                        'enable'
                    ],
                    'type': 'str'
                }
            }

        }
    }

    params_validation_blob = []
    check_galaxy_version(module_arg_spec)
    module = AnsibleModule(argument_spec=check_parameter_bypass(module_arg_spec, 'system_saml'),
                           supports_check_mode=False)

    fmgr = None
    if module._socket_path:
        connection = Connection(module._socket_path)
        fmgr = NAPIManager(jrpc_urls, perobject_jrpc_urls, module_primary_key, url_params, module, connection, top_level_schema_name='data')
        fmgr.validate_parameters(params_validation_blob)
        fmgr.process_partial_curd()
    else:
        module.fail_json(msg='MUST RUN IN HTTPAPI MODE')
    module.exit_json(meta=module.params)


if __name__ == '__main__':
    main()
